/*
 *        x
 *      -----> 
 *      0 1 2 3 4 5 6 7
 *  y  0 | | |0| | | | | |
 *  |  1 | | | | | |0| | |
 *  |  2 | | | | | | | |0|
 * \/  3 |0| | | | | | | |
 *     4 | | | | |0| | | |
 *     5 | | | | | | |0| |
 *     6 | |0| | | | | | |
 *     7 | | | |0| | | | |
 */


import java.util.Random;

/**
 * Class Sachovnice solves Eight queen puzzle by means of min conflicts algorithm.
 * 
 * @author Martin ehnek, 143357@mail.muni.cz
 * @version 23.11.2006
 */
public class Sachovnice
{
    private Pozice[] damy;
    private int[][] nulaJedna = new int[8][8];  //pole poli, tam kde nic neni je 0 a tam kde je dama je 1 
    
    
    /**
	 * Constructor for objects of class Sachovnice.
	 * @param damy Array of objects of class Pozice that represent positions of queens.
	 */
	public Sachovnice(Pozice[] damy)
    {
        this.damy = damy;
        for(int i = 0; i<8; i++)
          nulaJedna[damy[i].getX()][damy[i].getY()] = 1;
    }
    
    
    /**
	 * Nonparametric constructor for objects of class Sachovnice that creates random chessboard.
	 */
	public Sachovnice() //bezparametricky konstruktor, ktery generuje sachovnici nahodne.
    {
       int x = 0;
       int y = 0;
       damy = new Pozice[8];
       Random mujRandom = new Random();
       for(int i = 0; i<8; i++) //pro vsech 8 dam
         while (true)
          { y = mujRandom.nextInt(8);
            if (nulaJedna[x][y] == 0) 
              {
                  nulaJedna[x][y] = 1;
                  damy[i] = new Pozice(x,y);
                  x++;
                  break;
               }
           }
              
     }
    
    
    /**
     * Returns a string representation of the object.
     * @return A string representation of the object.
     */
    public String toString()  //chtel jsem to udelat Unicodem, at to vypada dobre, ale bohuzel
    {
        boolean barva = true;  //true bila, false cerna
        int i = 0;
        String navrat = "";
        for (i = 0; i<8; i++)
          {   //navrat+="-----------------\n";
              navrat+='|';
              for (int j = 0; j<8; j++)
                {                                       //prazdne policko     //policko s damou  
                    navrat+= (nulaJedna[j][i] == 0) ? (barva ? ' ' : ' ') : (barva ? '0' : '0');
                      navrat += '|';
                    barva = !barva;  
                 }
              navrat+='\n';
              barva = !barva;
           }
        //navrat+="-----------------\n";   
        return navrat;
     }     
    
    
 
     /**
      * Returns a number of conflicts with other queens that has the queen at chosen coordinates.
      * @return A number of conflicts with other queens that has the queen at chosen coordinates.
      * @param x X coordinate of the queen.
      * @param y Y coordinate of the queen.
      */
     private int konfliktu(int x, int y)  //prozkoumej nahoru dolu, doprava doleva, diagonalne, jestli neni konflikt 
     {   
          int j = 0;
          int k = 0;
          int pocet = 0;
          for(j = 0; j<8; j++)  //prozkoumat vodorovne
             {
                 if ( (j != x) && (nulaJedna[j][y] == 1)  )
                  {
                      pocet++; //System.out.print("V,");
                  }
                   
              }
           
          int newX = 0;
          int newY = 0;
          for(j = 1; j<7; j++) //prozkoumat diagonalne //mohlo by to byt v cyklu(ech), tim by ale dost klesla srozumitelnost
             {  
                // [+j][+j] 
               if ( ( (newX = x + j) >= 0) && (newX<8) && ( (newY = y + j) >= 0 ) && (newY<8) )   
                 if ((newX != x) && (newY != y) && (nulaJedna[newX][newY] == 1))
                   {pocet++; //System.out.print("D1,");
                    }
               // [+j][-j] 
               if ( ( (newX = x + j) >= 0) && (newX<8) && ( (newY = y - j) >= 0 ) && (newY<8) ) 
                 if ((newX != x) && (newY != y) && (nulaJedna[newX][newY] == 1))
                   {pocet++; //System.out.print("D2,");
                    }
               // [-j][+j] 
               if ( ( (newX = x - j) >= 0) && (newX<8) && ( (newY = y + j) >= 0 ) && (newY<8) ) 
                 if ((newX != x) && (newY != y) && (nulaJedna[newX][newY] == 1))
                   {pocet++; //System.out.print("D3,");
                   }
               // [-j][-j] 
               if ( ( (newX = x - j) >= 0) && (newX<8) && ( (newY = y - j) >= 0 ) && (newY<8) ) 
                 if ((newX != x) && (newY != y) && (nulaJedna[newX][newY] == 1))
                   {pocet++; //System.out.print("D4,");
                    }
             } 
          return pocet ;   
     }
     
     
     /**
      * Returns one of the queens that have least conflicts. It chooses one of them randomly. 
      * @return One of the queens that have least conflicts.
      */
     private Pozice vyberDamu() 
     {
         int i = 0;
         int[] damyKonfliktu = new int[8]; //kolik konfliktu ma i-ta dama z pole damy
         for(i = 0; i<8; i++)
           damyKonfliktu[i] = konfliktu(damy[i].getX(),damy[i].getY());
         
         //nahaz nekam damu s nejvetsi hodnotou konfliktu a nahodne vyber jednu z nich
         int max = 0;
         int[] maxHodnoty = new int[8];  //v nejhorsim pripade 8 dam stejne ohodnocenych
         int nejvyssich = 0;
         for(i = 0;  i<8; i++)  //nejvyse v konfliktu se 7 damami
            {
                if (damyKonfliktu[i] > max)
                  {max = damyKonfliktu[i];
                   nejvyssich = 1;
                   maxHodnoty[0] = i; }  //i-ta dama
                 else if (damyKonfliktu[i] == max)
                  {nejvyssich++;
                   maxHodnoty[nejvyssich - 1] = i;}
              }      
                      
         //nahodny vyber jedne damy na presun (z tech, ktere byly nejvic v konfliktu)
         Random mujRandom = new Random();
         return (max != 0) ? damy[maxHodnoty[mujRandom.nextInt(nejvyssich)]] : null; //same nuly -> null -> zadne damy nejsou v konfliktu
        
      }   
      
      
      /**
       * Returns position in the column to which it is best to move the queen. It decides according to possible conflicts on these positions. If there are more best positions the algorithm chooses randomly one of them.
       * @return Position in the column to which it is best to move the queen.
       * @param dama Queen whose new position is to be determined.
       */
      private int vyberKamDamu(Pozice dama)  //zjisti, na jakou pozici by bylo nejlepsi damu posunout (sloupec je pevny)
      {
        int j = 0;
        int[] konfliktuVRadku = new int[8];
        for (j = 0; j<8; j++) //vsechny pozice nad a pod nynejsi pozidi damy  
          konfliktuVRadku[j] = konfliktu(dama.getX(), j) ; 
           
        //Nahodny vyber jedne z nejmene ohodnocenych pozic
        int min = 7;
        int[] minHodnoty = new int[8];  //v nejhorsim pripade (teoreticky) 8 pozic stejne ohodnocenych (vcetne te soucasne)
        int nejmensich = 0;
        for(j = 0;  j<8; j++)  //nejvyse 8 ruznych pozic, kam se da vlezt
           {
             if (konfliktuVRadku[j] < min)
                  {min = konfliktuVRadku[j];
                   nejmensich = 1;
                   minHodnoty[0] = j; }  //i-ta dama
              else if (konfliktuVRadku[j] == min)
                  {nejmensich++;
                   minHodnoty[nejmensich - 1] = j;}
            }      
        Random mujRandom = new Random();
        return minHodnoty[mujRandom.nextInt(nejmensich)];
      }    
      
      
      /**
       * Sets y coordinate of the chosen queen.
       * @param dama Queen whose position is to be changed.
       * @param y New y coordinate of the queen.
       */
      private void setDamaY(Pozice dama, int y)
      {
          nulaJedna[dama.getX()][dama.getY()] = 0;
          dama.setY(y);
          nulaJedna[dama.getX()][y] = 1;
       }    
          
      
      /**
       * Moves the queens in their columns until it finds a nonconflict setting or overruns the maxCounter. Finally prints out nonconflict chessboard.
       * @param vypisy If true prints out chessboard after every step.
       * @param maxCounter Maximal number of steps after which computation is terminated.
       */
      public void Vyres(boolean vypisy, int maxCounter)
      { 
         
         int kamTedDamu = 0;
         int counter = 1;
         Pozice vybranaDama = null;
         System.out.println("Sachovnice:\n" + this);
            
         while( (vybranaDama = vyberDamu()) != null )
          {
            kamTedDamu = vyberKamDamu(vybranaDama);
            setDamaY(vybranaDama,kamTedDamu);
            if (vypisy)
              System.out.println("krok " + counter + ":\n" + this);
            //System.out.println("Vybrana dama: " + vybranaDama);
            //System.out.println("Damu ted dam na pozici " + kamTedDamu + '\n');
            
            counter++;
            if (counter == maxCounter)
             {
                 System.out.println("Ani po 1000 pokusech se nepodarilo vyresit");
                 return;
              }   
          }
         System.out.println("Problem se podarilo vyresit v " + (counter - 1) + " krocich"); 
         System.out.println("Vysledna sachovnice:\n" + this);
      }
      
      /**
       * Solves two chosen chessboard setting and one random.
       */
      public static void main()
       {
         Pozice[] prednaskaDamy1 = {new Pozice(0,1),new Pozice(1,4),new Pozice(2,6),new Pozice(3,3),new Pozice(4,0),new Pozice(5,2),new Pozice(6,5),new Pozice(7,7) } ;
         Pozice[] prednaskaDamy2 = {new Pozice(0,0),new Pozice(1,1),new Pozice(2,2),new Pozice(3,3),new Pozice(4,4),new Pozice(5,5),new Pozice(6,6),new Pozice(7,7) } ;
         //Pozice[] damy1 = {new Pozice(0,7),new Pozice(1,4),new Pozice(2,3),new Pozice(3,6),new Pozice(4,5),new Pozice(5,0),new Pozice(6,4),new Pozice(7,6) } ;
         //Pozice[] damy2 = {new Pozice(0,6),new Pozice(1,4),new Pozice(2,0),new Pozice(3,0),new Pozice(4,3),new Pozice(5,7),new Pozice(6,0),new Pozice(7,1) } ;
         
         
         Sachovnice easy = new Sachovnice(prednaskaDamy1);
         easy.Vyres(false,1000); //pro zobrazeni vypisu prepsat na true 
         Sachovnice diagonala = new Sachovnice(prednaskaDamy2);
         diagonala.Vyres(false,1000); 
         Sachovnice nahoda = new Sachovnice(); //konstruktor bez paramatru vygeneruje nahodnou sachovnici
         nahoda.Vyres(false,1000); 
       }
    

}

   
